program mpcenct;
// Copyright (c) 2002 Erik Stenborg (erik.stenborg@home.se)

{$APPTYPE CONSOLE}
uses
  SysUtils, Windows, Apetag;

const
  Version = 'MPCENCT v0.32    Copyright (c) 2002 Erik Stenborg (erik.stenborg@home.se)';
var
  cmd, infile, outfile, errfile:string;
  n,i,v,e: integer;
  tags, ape: TApetag;
  verbose, HasConsole: boolean;
  f: file;
  buf: array [1..1024] of Char;
  StdErr: TextFile;

function RunAndWait(arg:String; Visibility : integer):dword;
var
  zAppName:pchar;
  StartupInfo:TStartupInfo;
  ProcessInfo:TProcessInformation;
begin
  zAppName := PChar(arg);
  FillChar(StartupInfo,Sizeof(StartupInfo),#0);
  StartupInfo.cb := Sizeof(StartupInfo);
  StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow := Visibility;
  if not CreateProcess(nil, zAppName, nil, nil, true, NORMAL_PRIORITY_CLASS, nil, nil, StartupInfo, ProcessInfo) then Result := 0
  else begin
    WaitforSingleObject(ProcessInfo.hProcess,INFINITE);
    GetExitCodeProcess(ProcessInfo.hProcess,Result);
    CloseHandle(ProcessInfo.hProcess);
    CloseHandle(ProcessInfo.hThread);
  end;
end;

begin
  try
    //is stdout writable?
{$I-}
    Rewrite(Output);
{$I+}
    HasConsole := (IOResult = 0);
    Move(output, stderr, Sizeof(stderr));
    TTextRec(stderr).Handle := GetStdhandle(STD_ERROR_HANDLE);
    //If no parameters: Write info text end halt
    if paramcount < 1 then begin
      if HasConsole then begin
        writeln(stderr, Version);
        writeln(stderr, 'Encode Musepack files and add Ape Tags when the encoding finish.');
        writeln(stderr, 'Useful in EAC, CDex, Audiograbber etc.');
        writeln(stderr, 'Example: mpcenct.exe <normal mppenc parameters> --tags "Key=Value" "Apa=bepa"');
      end;
      halt(0);
    end;
    //initialize variables
    cmd := 'mppenc.exe';
    v := 0;
    infile := '';
    outfile := '';
    e := -1;
    verbose := false;
    tags := TApetag.Create;
    //check all the parameters
    for i := 1 to paramcount do begin
      if v <> 0 then begin
        //this parameter is a tag
        n := pos('=', paramstr(i));
        tags.SetValue(copy(paramstr(i), 1, n-1), copy(paramstr(i), n+1, length(paramstr(i))-n));
      end else begin
        if paramstr(i) = '--tags' then v := 1
        else if paramstr(i) = '--tags2' then v := 2
        else if paramstr(i) = '--stderr' then e := i + 1
        else if i <> e then begin
          //this parameter should be sent to mppenc
          cmd := cmd + ' ' + AnsiQuotedStr(paramstr(i), '"');
          if paramstr(i) = '--verbose' then verbose := true;
          if paramstr(i) = '--silent' then verbose := false;
          infile := outfile;
          outfile := paramstr(i);
        end;
      end;
    end;
    //Clear incorrect year values (added by EAC)
    try
      If not VarIsEmpty(tags.GetValue('Year')) then
        if tags.GetValue('Year') <= 0 then
          tags.SetValue('Year', '');
    except
    end;
    //if only one filename is specified on the command line
    if (UpperCase(ExtractFileExt(outfile)) <> '.MPC') and (outfile <> '-') then begin
      infile := outfile;
      outfile := ChangeFileExt(infile, '.mpc');
    end;
    //if the input file is an ape file then just transfer the tags
    if (UpperCase(ExtractFileExt(infile)) = '.APE') then begin
      ape := tags;
      tags := TApeTag.Create;
      tags.ReadFromFile(infile);
      for i := 0 to ape.Count-1 do
        tags.SetValue(ape.Keys[i], ape.Values[i]);
      ape.Free;
    end;
    if HasConsole then
      WriteLn(stderr, Version + #10#13)
    else
      verbose := verbose and (e <> -1);
    //redirect all info to the file given after --stderr
    if e <> -1 then begin
      if ExtractFilePath(paramstr(e)) = '' then
        errfile := ExtractFilePath(outfile) + paramstr(e)
      else
        errfile := paramstr(e);
      Insert(' --stderr ' + AnsiQuotedStr(errfile, '"'), cmd, 11);
      if HasConsole then CloseFile(stderr);
      AssignFile(stderr, errfile);
{$I-}
      Append(stderr);
{$I+}
      if IOResult <> 0 then Rewrite(stderr);
    end;
    //print some debug info
    if verbose and (hasConsole or (e <> -1)) then
    begin
      //writeln(stderr, 'infile: ' + infile);
      //writeln(stderr, 'outfile: ' + outfile);
      writeln(stderr, 'mppenc string: ' + cmd);
      if v <> 0 then begin
        writeln(stderr, 'Tag type: APE Tag v' + inttostr(v));
        writeln(stderr, '--- Tags ----------------------------------------------------------------------');
        for i := 0 to tags.Count-1 do
          WriteLn(stderr, tags.Keys[i], StringOfChar(' ', 12 - Length(tags.Keys[i])), ' = ', tags.Values[i]);
      end else
        writeln(stderr, 'Tag type: None');
      writeln(stderr, '--- Mppenc output -------------------------------------------------------------');
    end;
    if e <> -1 then CloseFile(stderr);
    //start mppenc and wait for it to exit
    runandwait(cmd, 0);
    tags.Version := v;
    if v in [1, 2] then
      if outfile = '-' then begin
        //print the tag to stdout
        AssignFile(f, 'tag.tmp');
        Rewrite(f);
        tags.SaveToFile('tag.tmp');
        CloseFile(f);
        AssignFile(f, 'tag.tmp');
        Reset(f,1);
        while not EOF(f) do begin
          BlockRead(f, buf, SizeOf(buf), n);
          for i := 1 to n do
            Write(buf[i]);
        end;
        CloseFile(f);
        DeleteFile(PChar('tag.tmp'));
      end else
        //append the tag to the end of the outputfile
        tags.SaveToFile(outfile);
    tags.Free;

  except
    on E: Exception do begin
      WriteLn(E.Message);
      //pause so that the user has time to read what went wrong
      WriteLn('Press Enter to exit');
      ReadLn;
      halt(1);
    end;
  end;

end.
